/*
 * This file is part of the Alliance CAD System
 * Copyright (C) Laboratoire LIP6 - Dpartement ASIM
 * Universite Pierre et Marie Curie
 * 
 * Home page          : http://www-asim.lip6.fr/alliance/
 * E-mail support     : mailto:alliance-support@asim.lip6.fr
 * 
 * This progam is  free software; you can redistribute it  and/or modify it
 * under the  terms of the GNU  General Public License as  published by the
 * Free Software Foundation;  either version 2 of the License,  or (at your
 * option) any later version.
 * 
 * Alliance VLSI  CAD System  is distributed  in the hope  that it  will be
 * useful, but WITHOUT  ANY WARRANTY; without even the  implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy  of the GNU General Public License along
 * with the GNU C Library; see the  file COPYING. If not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*********************************************************************\

    File        :    dpr.c

    Date        :   30-06-94
    Version     :   1

    Auteur      :   BEAURIN oliver

    Description :   Main function for router for datapath compiler

\*********************************************************************/

/*---------------------------------------------------------\
                     includes
\---------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>

#include MUT_H
#include MLO_H
#include MPH_H

#include GFP_H

#include "dpr.h"
#include "placer_main.h"

/*---------------------------------------------------------\
                     defines
\---------------------------------------------------------*/
#define F_VERBOSE     1
#define F_VERBOSE_ALL 2
#define F_PLACER      4
#define F_ROUTER      8
#define F_DP_WIDTH   16
#define F_NB_ITE     32
#define F_CELLS_COLS 64
#define F_OPTIMIZER 128
#define F_STRECH    256

#define DEBUG_VAR "DEBUG_DPR"

/*---------------------------------------------------------\
                     globals variables
\---------------------------------------------------------*/
char verbose_mode = FALSE;
char debug_mode   = FALSE;

DPR_Var_t  DPR_Var = {
                 ALU2,  /* Layer par defaul des alimentations.      */
                    7,  /* Premiere piste a partir du haut de l'AB. */
                    8   /* Premiere piste a partir du bas de l'AB.  */
                     };

/*---------------------------------------------------------\
                     usage
\---------------------------------------------------------*/
void usage( progname )
char *progname;
{
    fprintf( stderr, "Usage : %s [-v|-V] [-p] [-s name1 s:t[,s1:t1]] [-r name2] [-w Nb] [-i nb] [-c nb] source\n", progname );
    fprintf( stderr, "\n" );
    fprintf( stderr, "    -v sets verboes mode normal\n" );
    fprintf( stderr, "    -V sets verbose mode all\n" );
    fprintf( stderr, "\n" );
    fprintf( stderr, "    -p activate placement step\n" );
    fprintf( stderr, "    -r activate routing step where name2 is the routed layout name\n" );
    fprintf( stderr, "    -o activate placement optimizer in routing step\n" );
    fprintf( stderr, "    -s strech slice s of t track where name1 is the new netlist and placed layout name\n" );
    fprintf( stderr, "\n" );
    fprintf( stderr, "    -w sets datapath minimum width for placement step\n" );
    fprintf( stderr, "    -i sets the default iteration number for glue logic placement\n" );
    fprintf( stderr, "    -c sets the default number of cells put on one column in glue logic bloc\n" );
    fprintf( stderr, "\n" );
    fprintf( stderr, "    source : name of netlist to place/rout\n" );
    fprintf( stderr, "\n" );
    exit( 1 );
} /* fin de usage */

/*---------------------------------------------------------\
                     main
\---------------------------------------------------------*/
int 
main( argc, argv )
int argc;
char **argv;
{
    int   i;
    int   arg;
    int   layout_index;
    long  dp_width    = 0L;
    long  flags_arg   = 0x00;
    long  nb_cells    = 1L;
    long  nb_ite      = 100L;
    char *strech_name = NULL;

    alliancebanner ("DPR", DPR_VERSION, "Placer-Router for Datapath Compiler", "1994", ALLIANCE_VERSION);
    debug_mode = ( getenv( DEBUG_VAR ) != NULL );

    if( argc < 2 )
    {
        usage( argv[ 0 ] );
    }

    for( arg = 1; arg < argc - 1; arg++ )
    {
        if( strcmp( argv[ arg ], "-p" ) == 0 )
        {
            if( flags_arg & F_PLACER )
                usage( argv[ 0 ] );
            flags_arg |= F_PLACER;
        }
        else if( strcmp( argv[ arg ], "-o" ) == 0 )
        {
            if( flags_arg & F_OPTIMIZER )
                usage( argv[ 0 ] );
            flags_arg |= F_OPTIMIZER;
        }
        else if( strcmp( argv[ arg ], "-r" ) == 0 )
        {
            if( flags_arg & F_ROUTER )
                usage( argv[ 0 ] );
            arg++;
            if( arg == ( argc - 1 ) )
                usage( argv[ 0 ] );
            flags_arg |= F_ROUTER;
            layout_index = arg;
        }
        else if( strcmp( argv[ arg ], "-s" ) == 0 )
        {
            if( flags_arg & F_STRECH )
                usage( argv[ 0 ] );
            if( arg >= ( argc - 3 ) )
                usage( argv[ 0 ] );
            flags_arg |= F_STRECH;
            if( ! parse_strech_param( argv[ arg + 2 ] ) )
            {
                usage( argv[ 0 ] );
            }
            strech_name = argv[arg + 1];
            arg += 2;
        }
        else if( strcmp( argv[ arg ], "-v" ) == 0 )
        {
            if( ( flags_arg & F_VERBOSE )
              ||( flags_arg & F_VERBOSE_ALL ) )
                usage( argv[ 0 ] );
            flags_arg |= F_VERBOSE;
            verbose_mode = VERBOSE_NORMAL;
        }
        else if( strcmp( argv[ arg ], "-V" ) == 0 )
        {
            if( ( flags_arg & F_VERBOSE )
              ||( flags_arg & F_VERBOSE_ALL ) )
                usage( argv[ 0 ] );
            flags_arg |= F_VERBOSE_ALL;
            verbose_mode = VERBOSE_ALL;
        }
        else if( strcmp( argv[ arg ], "-w" ) == 0 )
        {
            if( flags_arg & F_DP_WIDTH )
                usage( argv[ 0 ] );
            arg++;
            if( arg == argc - 1 )
                usage( argv[ 0 ] );
            flags_arg |= F_DP_WIDTH;
            dp_width = atol( argv[ arg ] );
            if( dp_width <= 0 )
            {
                fprintf( stderr, "*** dpr error *** : Datapath width must be positive.\n" );
                exit( 1 );
            }
        }
        else if( strcmp( argv[ arg ], "-i" ) == 0 )
        {
            if( flags_arg & F_NB_ITE )
                usage( argv[ 0 ] );
            arg++;
            if( arg == ( argc - 1 ) )
                usage( argv[ 0 ] );
            flags_arg |= F_NB_ITE;
            nb_ite = atol( argv[ arg ] );
        }
        else if( strcmp( argv[ arg ], "-c" ) == 0 )
        {
            if( flags_arg & F_CELLS_COLS )
                usage( argv[ 0 ] );
            arg++;
            if( arg == ( argc - 1 ) )
                usage( argv[ 0 ] );
            flags_arg |= F_CELLS_COLS;
            nb_cells = atol( argv[ arg ] );
        }
        else
        {
            usage( argv[ 0 ] );
        }
    }

    if( ( !( flags_arg & F_PLACER ) ) && ( !( flags_arg & F_ROUTER ) ) )
    {
        fprintf( stderr, "One of '-p' or '-r' option must be selected.\n" );
        exit( 1 );
    }

    mbkenv( );

	UtBas_Init();
	UtEnv_Init();
    UtTec_Init( UtEnv_Environ.RDS_TECHNO_NAME );
	UtUsr_Init();
	UtFlt_Init();
	UtMLo_Init();
	UtPwB_Init();
	UtHir_Init();
	UtFDP_Init();
	PlCon_Init();

	switch(UtTec_V.Type)
	{
		case C_UtTec_TypeAsGa3:
			DPR_Var.LAYER_ALIM = ALU3;
			DPR_Var.TOP_TRACK  = 6;
			DPR_Var.BOT_TRACK  = 6;
			break;
		case C_UtTec_TypeCMOS2:
		case C_UtTec_TypeUnknow:
		default:
			DPR_Var.LAYER_ALIM = ALU2;
			DPR_Var.TOP_TRACK  = 7;
			DPR_Var.BOT_TRACK  = 8;
			break;
	}

    if( verbose_mode == VERBOSE_ALL )
    {
        fprintf( stdout, "\n"                                      );
        fprintf( stdout, " * MBK parameters :\n"                   );
        fprintf( stdout, "\n"                                      );
        fprintf( stdout, "       MBK_WORK_LIB   : %s\n", WORK_LIB  );
        for( i = 0; CATA_LIB[ i ]; i++ )
            if( i == 0 )
                fprintf( stdout, "       MBK_CATA_LIB   : %s\n", CATA_LIB[ i ]  );
            else
                fprintf( stdout, "                      : %s\n", CATA_LIB[ i ]  );
        fprintf( stdout, "       MBK_IN_PH      : %s\n", IN_PH     );
        fprintf( stdout, "       MBK_OUT_PH     : %s\n", OUT_PH    );
        fprintf( stdout, "       MBK_IN_LO      : %s\n", IN_LO     );
        fprintf( stdout, "       MBK_OUT_LO     : %s\n", OUT_LO    );
        fprintf( stdout, "       TECHNO         : %s\n", (UtTec_V.Type == C_UtTec_TypeAsGa3) ? "ASGA" : "CMOS" );
        fprintf( stdout, "\n"                                      );
        fflush( stdout );
    }

    if( flags_arg & F_PLACER )
    {
        placer( argv[ argc - 1 ], strech_name, dp_width, nb_ite, nb_cells );
    }

    if( flags_arg & F_ROUTER )
    {
        if( flags_arg & F_PLACER )
        {
            fprintf( stdout, "\n" );
            fflush( stdout );
        }
        router( strech_name ? strech_name : argv[ argc - 1 ], argv[ layout_index ], (flags_arg & F_OPTIMIZER) ? TRUE : FALSE );
    }

    return( 0 );
} /* fin de main */
