/*
 * This file is part of the Alliance CAD System
 * Copyright (C) Laboratoire LIP6 - Dpartement ASIM
 * Universite Pierre et Marie Curie
 * 
 * Home page          : http://www-asim.lip6.fr/alliance/
 * E-mail support     : mailto:alliance-support@asim.lip6.fr
 * 
 * This progam is  free software; you can redistribute it  and/or modify it
 * under the  terms of the GNU  General Public License as  published by the
 * Free Software Foundation;  either version 2 of the License,  or (at your
 * option) any later version.
 * 
 * Alliance VLSI  CAD System  is distributed  in the hope  that it  will be
 * useful, but WITHOUT  ANY WARRANTY; without even the  implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy  of the GNU General Public License along
 * with the GNU C Library; see the  file COPYING. If not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*********************************************************************\

	File		:	flop.c

	Date		:	14-12-93
	Version		:	0.0

	Author		:	PICAT Xavier

	Description :	Channel optimizer

\*********************************************************************/
/*---------------------------------------------------------\
                   Includes
\---------------------------------------------------------*/
#include MUT_H
#include MPH_H
#include MLO_H
#include MPU_H
#include MLU_H
#include <malloc.h>
#include <stdio.h>
#include <string.h>

/*---------------------------------------------------------\
                   Defines
\---------------------------------------------------------*/
#define DEBUG
#ifdef DEBUG
#	define PRINT(x) printf x
#	define TPRINT(x,y) if(x) printf y
#	define XPRINT(cdt)   if (cdt) printf ("x=%d, y=%d, x1 = %d\n",x,y,x1)
#else  /* DEBUG */
#	define PRINT(x)
#	define TPRINT(x)
#	define XPRINT(x)
#endif /* DEBUG */
#define SAVE(x) do {if (x) { SaveArray("c2"); exit(0); }}while (0)

#define max(a,b) ((a)>(b)?(a):(b))
#define min(a,b) ((a)<(b)?(a):(b))
#define EMPTY      0
#define T_ALU1     (1<<0)
#define T_ALU2     (1<<1)
#define T_VIA      (1<<2)

/* This value implies that for every ALU1 wire abs(x1-x2)>=RULE && abs(y1-y2)>=RULE */
#define RULE       4
enum {FALSE, TRUE};

/*---------------------------------------------------------\
                   Local Vars
\---------------------------------------------------------*/
static phfig_list *ptChannel; /* Physical description of the channel */
static int         Width;     /* Width  of the channel */
static int         Height;    /* Height of the channel */
static int         Ymin;      /* First ordinates */
static int         Ymax;      /* Last  ordinates */
static int         StartAlu2; /* First vertical ALU2 wire */
static int         EndAlu2;   /* End   vertical ALU2 wire */
static char      **Array;     /* Array representation of the channel */
static chain_list *WestCons;  /* List of west connectors */
static chain_list *EastCons;  /* List of east connectors */

/*---------------------------------------------------------\
                   Local Functions
\---------------------------------------------------------*/
/*static void OptimizeChannel (char *SourceFile, char *TargetFile);*/
static void OptimizeArray   (void);
static void OptimizeViaUp   (int x, int y, int x1);
static void OptimizeViaDown (int x, int y, int x1);
static void LoadArray       (phfig_list *);
static phfig_list *SaveArray(long, long, char *FileName);
static int  CheckUp         (int x, int y, int x1);
static int  CheckDown       (int x, int y, int x1);

static void MoveNotchUp     (int x, int y, int x1);
static void MoveNotchDown   (int x, int y, int x1);
static void MoveWireUp      (int x, int y, int x1, int y1);
static void MoveWireDown    (int x, int y, int x1, int y1);

/*---------------------------------------------------------\
					main
\---------------------------------------------------------*/
#if 0
int main(int argc, char **argv)
{
	if (argc != 3)
	{
		fprintf (stderr, "usage : %s <SourceFile> <TargetFile>\n", argv[0]);
		exit (1);
	}
	mbkenv ();
	OptimizeChannel (argv[1], argv[2]);
	return (0);
} /* end of main */
#endif

/*---------------------------------------------------------\
					OptimizeChannel 
\---------------------------------------------------------*/
phfig_list *OptimizeChannel (ptChannel, FileName)
phfig_list *ptChannel;
char *FileName;
{
long X1, Y1;
   X1 = ptChannel->XAB1;
   Y1 = ptChannel->YAB1;
	LoadArray (ptChannel);
	OptimizeArray ();
	return SaveArray (X1, Y1, FileName);
} /* end of OptimizeChannel  */

/*---------------------------------------------------------\
					OptimizeArray   
\---------------------------------------------------------*/
static void OptimizeArray (void)
{
	int x, y, x0, x1;

	/* We cannot have a notch at y=0 or y=Height, because it wouldn't respect half-rule */
	for (x=0; x<Width; x++)
	{
		for (y=Ymin; y<=Ymax; y++)
			if ( (Array[x][y]   & T_ALU1) &&
			     (Array[x+1][y] & T_ALU1)
			   )
			{
				/* Computing x0 is very important because we may move a wire
				 * which starts on a VIA, located before this notch that we
				 * may have created in a previous step
				 */
				x0 = x-1;
				while (x0>=0 && (Array[x0][y]&T_ALU1))
					x0--;
				x0++;
				x1 = x+2;
				while (x1<Width && (Array[x1][y]&T_ALU1))
					x1++;
				x1--;
				if (Array[x1][y+1] & T_ALU1)
					MoveNotchUp (x0,y,x1);
				else if (Array[x1][y-1] & T_ALU1)
					MoveNotchDown (x0,y,x1);
			}
	}
} /* end of OptimizeArray */

/*---------------------------------------------------------\
					MoveNotchUp

Description : This procedure is called when we know that a
              notch exists on the (x,y) position. It tries
			  to optimize as much as possible till the first
			  via. If we create a new notch, this procedure
			  calls itself.
\---------------------------------------------------------*/
static void MoveNotchUp (int x, int y, int x1)
{
	int FirstX, i;

	/*                      <--- not always a VIA here because we can generate additionnal notchs
	 *                    |      in order to apply the minimal design rule.
	 *                    |
	 * y          ---------
	 *
	 *            x       x1
	 */

	/* If (x,y) is on a VIA, we must take care not to create a notch */
	if (Array[x][y] & T_VIA)
	{
		int i;

		do
		{
			for (i=y; (i < y+RULE) && CheckUp (x,i,x1); i++);
			if (i < y+RULE)
				x++;
		} while (x<x1 && i < y+RULE);
		if (x>=x1)
			return;
	}
	do
	{
		FirstX = x;
		i = y;
		while ( (Array[x1][y+1]&T_ALU1) &&
				 CheckUp (x, y, x1)     &&
				 y < EndAlu2
			  )
		{
			y++;
		}
		MoveWireUp (x, i, x1, y);
		if ( y >= EndAlu2) /* Don't generate notch over routing area */
			return;
		if (Array[x1-1][y+RULE]&T_ALU1)
		{
			int OtherNotchStart;

			OtherNotchStart = x1-1;
			while ( (Array[OtherNotchStart][y+RULE]&T_ALU1) &&
			        OtherNotchStart >= x)
				OtherNotchStart--;
			OtherNotchStart++;
			for (i=OtherNotchStart+1; i<x1; i++)
			{
				Array[i][y]      &= ~T_ALU1;
				Array[i][y+RULE] &= ~T_ALU1;
			}
			for (i=y; i <= y+RULE; i++)
			{
				/* ATTENTION : x1 may equal OtherNotchStart, so remove ALU1 first then add last */
				Array[x1][i]              &= ~T_ALU1;
				Array[OtherNotchStart][i] |=  T_ALU1;
			}
			x1 = OtherNotchStart;
			if (x1==x)
				return;
		}

		/* We can't put an ALU1 wire in x,y;
		 * but may be we can in x+1,y ...
		 */
		while ( (x<x1) && !CheckUp (x,y,x1) )
			x++;
	} while (x < x1);

	/* Now we will look why we stop here */
	OptimizeViaUp (FirstX, y, x1);
} /* end of MoveNotchUp */

/*---------------------------------------------------------\
					OptimizeViaUp
\---------------------------------------------------------*/
static void OptimizeViaUp (int x, int y, int x1)
{
	int  YVia, i, X, Xup;
	char Value;

	YVia = y+RULE;
	if ( !(Array[x1][YVia] & T_VIA) )
		return;

	/* Looking how much we could move the VIA left */
	for (X = max (0,x1-RULE); X > max(0,x-RULE); X--)
	{
		if (Array[X-1][YVia] & T_VIA)
			break;
		for (i = y+1; i < YVia+RULE; i++)
			if (Array[X][i] & T_ALU1)
				break;
		if (i < YVia+RULE)
			break;
	}
	X += RULE;
	if ( x1 == X )
		return;
	if ( (Array[x1][YVia+1]      & T_ALU1) &&
	     (Array[x1-1][YVia+RULE] & T_ALU1)
	   )
	{
		/* Same hook over the VIA, looking for the end of it */
		Xup = x1-2;
		while (Array[Xup][YVia+RULE] & T_ALU1)
			Xup--;
		Xup++;
		if (X > Xup)
			Xup = X;
		for (i = Xup+1; i <= x1; i++)
			Array[i][YVia+RULE] &= ~T_ALU1;
		for (i = YVia; i < YVia+RULE; i++)
		{
			Array[Xup][i] |=  T_ALU1;
			Array[x1][i]  &= ~T_ALU1;
		}
	}
	/* Moving wire from right to left */
	Array[x1][YVia] &= ~T_VIA;
	Array[X][YVia]  |=  T_VIA|T_ALU1|T_ALU2;
	for (i = y; i <= YVia; i++)
	{
		Array[X][i]  |=  T_ALU1;
		Array[x1][i] &= ~T_ALU1;
	}
	for (i = X+1; i < x1; i++)
		Array[i][y] &= ~T_ALU1;
	Value = EMPTY;
	if (Array[x1+1][YVia] & T_ALU2)
	{
		Value            = T_ALU2;
		Array[x1][YVia] |= T_ALU2;
	}
	else
		Array[x1][YVia] &= ~T_ALU2;
	if ((Array[x1][YVia+1]   & T_ALU1) ||
	    (Array[x1+1][YVia]   & T_ALU1) ||
		(Array[x1-1][YVia+1] & T_ALU1)
	   )
	{
		if ( !(Array[x1-1][YVia+1] & T_ALU1) )
			Array[x1][YVia] |= T_ALU1;
		Value |= T_ALU1;
	}
	for (i = X+1; i < x1; i++)
		Array[i][YVia] = Value;
} /* end of OptimizeViaUp */

/*---------------------------------------------------------\
					MoveWireUp
\---------------------------------------------------------*/
static void MoveWireUp (int x, int y, int x1, int y1)
{
	int i;

	for (i=y; i<=y1; i++)
	{
		Array[x1][i] &= ~T_ALU1;
		Array[x][i]  |=  T_ALU1;
	}
	for (i=x+1; i<=x1; i++)
	{
		Array[i][y]  &= ~T_ALU1;
		Array[i][y1] |=  T_ALU1;
	}
} /* end of MoveWireUp */

/*---------------------------------------------------------\
					CheckUp
\---------------------------------------------------------*/
static int CheckUp (int x, int y, int x1)
{
	/* we want to put an alu1 segment in x,y+1 respecting designs rules
	 * We know that an ALU1 wire already exists in x,y => (supposing RULE==4)
	 * 'A'lu1, 'E'mpty
	 * 
	 *  E E E E E E
	 *  E E E E E E
	 *  E E E E E E
	 *  E E E A A A
	 *
	 * So we just have to check if the y+RULE line is empty
	 * When we reach the abutment box, we must only respect HALF-RULE
	 */

	y++;
	if ( y >= Height-(RULE>>1) )
		/* We won't respect HALF-RULE */
		return FALSE;
	y += RULE-1;
	if ( y >= Height )
		return TRUE;

	for (x = max(0,x-RULE+1); x < x1; x++)
		if (Array[x][y] & (T_ALU1 | T_VIA))
			return FALSE;
	if ((Array[x1][y] & T_VIA))
		return FALSE;
	return TRUE;
} /* end of CheckUp */

/*---------------------------------------------------------\
					MoveNotchDown

Description : This procedure is called when we know that a
              notch exists on the (x,y) position. It tries
			  to optimize as much as possible till the first
			  via. If we create a new notch, this procedure
			  calls itself.
\---------------------------------------------------------*/
static void MoveNotchDown (int x, int y, int x1)
{
	int FirstX, i;

	/*
	 * y          ---------
	 *                    |
	 *                    |
	 *                      <--- not always a VIA here because we can generate additionnal notchs
	 *            x       x1
	 */
	/* If (x,y) is on a VIA, we must take care not to create a notch */
	if (Array[x][y] & T_VIA)
	{
		int i;

		do
		{
			for (i=y; (i > y-RULE) && CheckDown (x,i,x1); i--);
			if (i > y-RULE)
				x++;
		} while (x<x1 && i > y-RULE);
		if (x>=x1)
			return;
	}
	do
	{
		FirstX = x;
		i = y;
		while ( (Array[x1][y-1]&T_ALU1) &&
				 CheckDown (x, y, x1)   &&
				 y > StartAlu2
			  )
		{
			y--;
		}
		MoveWireDown (x, i, x1, y);
		if ( y <= StartAlu2 ) /* Don't generate notch over routing area */
			return;
		if (Array[x1-1][y-RULE]&T_ALU1) /* we met a notch we generate during up pass */
		{
			int OtherNotchStart;

			OtherNotchStart = x1-1;
			while ( (Array[OtherNotchStart][y-RULE]&T_ALU1) &&
			        OtherNotchStart >= x)
				OtherNotchStart--;
			OtherNotchStart++;
			for (i=OtherNotchStart+1; i<x1; i++)
			{
				Array[i][y]      &= ~T_ALU1;
				Array[i][y-RULE] &= ~T_ALU1;
			}
			for (i=y-RULE; i <= y; i++)
			{
				/* ATTENTION : x1 may equal OtherNotchStart, so remove ALU1 first then add last */
				Array[x1][i]              &= ~T_ALU1;
				Array[OtherNotchStart][i] |=  T_ALU1;
			}
			x1 = OtherNotchStart;
			if (x1==x)
				return;
		}

		/* We can't put an ALU1 wire in x,y;
		 * but may be we can in x+1,y ...
		 */
		while ( (x<x1) && !CheckDown (x,y,x1) )
			x++;
	} while (x < x1);

	/* Now we will look why we stop here */
	OptimizeViaDown (FirstX, y, x1);
} /* end of MoveNotchDown */

/*---------------------------------------------------------\
					OptimizeViaDown
\---------------------------------------------------------*/
static void OptimizeViaDown (int x, int y, int x1)
{
	int  YVia, i, X, Xup;
	char Value;

	YVia = y-RULE;
	if ( !(Array[x1][YVia] & T_VIA) )
		return;

	/* Looking how much we could move the VIA left */
	for (X = max (0,x1-RULE); X > max(0,x-RULE); X--)
	{
		if (Array[X-1][YVia] & T_VIA)
			break;
		for (i = YVia-RULE+1; i < y; i++)
			if (Array[X][i] & T_ALU1)
				break;
		if (i < y)
			break;
	}
	X += RULE;
	if ( x1 == X )
		return;
	if ( (Array[x1][YVia-1]      & T_ALU1) &&
	     (Array[x1-1][YVia-RULE] & T_ALU1)
	   )
	{
		/* Same hook over the VIA, looking for the end of it */
		Xup = x1-2;
		while (Array[Xup][YVia-RULE] & T_ALU1)
			Xup--;
		Xup++;
		if (X > Xup)
			Xup = X;
		for (i = Xup+1; i <= x1; i++)
			Array[i][YVia-RULE] &= ~T_ALU1;
		for (i = YVia-RULE+1; i <= YVia; i++)
		{
			Array[Xup][i] |=  T_ALU1;
			Array[x1][i]  &= ~T_ALU1;
		}
	}
	/* Moving wire from right to left */
	Array[x1][YVia] &= ~T_VIA;
	Array[X][YVia]  |=  T_VIA|T_ALU1|T_ALU2;
	for (i = YVia; i <= y; i++)
	{
		Array[X][i]  |=  T_ALU1;
		Array[x1][i] &= ~T_ALU1;
	}
	for (i = X+1; i < x1; i++)
		Array[i][y] &= ~T_ALU1;
	Value = EMPTY;
	if (Array[x1+1][YVia] & T_ALU2)
	{
		Value            = T_ALU2;
		Array[x1][YVia] |= T_ALU2;
	}
	else
		Array[x1][YVia] &= ~T_ALU2;
	if ((Array[x1][YVia-1] & T_ALU1) ||
	    (Array[x1+1][YVia] & T_ALU1)
	   )
	{
		Array[x1][YVia] |= T_ALU1;
		Value |= T_ALU1;
	}
	if ( (Array[x1-1][YVia-1] & T_ALU1) )
		Value |= T_ALU1;
	for (i = X+1; i < x1; i++)
		Array[i][YVia] = Value;
} /* end of OptimizeViaDown */

/*---------------------------------------------------------\
					CheckDown
\---------------------------------------------------------*/
static int CheckDown (int x, int y, int x1)
{
	/* we want to put an alu1 segment in x,y-1 respecting designs rules
	 * We know that an ALU1 wire already exists in x,y => (supposing RULE==4)
	 * 'A'lu1, 'E'mpty
	 * 
	 *  A A A E E E
	 *  E E E E E E
	 *  E E E E E E
	 *  E E E E E E
	 *
	 * So we just have to check if the y+RULE line is empty
	 * When we reach the abutment box, we must only respect HALF-RULE
	 */

	y--;
	if ( y <= (RULE>>1) )
		/* We won't respect HALF-RULE */
		return FALSE;
	y -= RULE-1;
	if ( y < 0 )
		return TRUE;

	for (x = max(0,x-RULE+1); x < x1; x++)
		if (Array[x][y] & (T_ALU1 | T_VIA))
			return FALSE;
	if ((Array[x1][y] & T_VIA))
		return FALSE;
	return TRUE;
} /* end of CheckDown */

/*---------------------------------------------------------\
					MoveWireDown
\---------------------------------------------------------*/
static void MoveWireDown (int x, int y, int x1, int y1)
{
	int i;

	for (i=y; i>=y1; i--)
	{
		Array[x1][i] &= ~T_ALU1;
		Array[x][i]  |=  T_ALU1;
	}
	for (i=x+1; i<=x1; i++)
	{
		Array[i][y]  &= ~T_ALU1;
		Array[i][y1] |=  T_ALU1;
	}
} /* end of MoveWireUp */

/*---------------------------------------------------------\
					LoadArray
\---------------------------------------------------------*/
static void LoadArray (ptChannel)
phfig_list *ptChannel;
{
	phseg_list *ptSeg;
	phvia_list *ptVia;
	int         X1, Y1, i;

	/* Reading channel dimensions */
	X1 = ptChannel->XAB1;
	Y1 = ptChannel->YAB1;

	/* Setting bottom left coordinate of the abutment box to zero */
	/* Dividing all distances and positions by SCALE_X            */
	ptChannel->XAB1 = 0;
	ptChannel->YAB1 = 0;
	ptChannel->XAB2 = (ptChannel->XAB2-X1)/SCALE_X;
	ptChannel->YAB2 = (ptChannel->YAB2-Y1)/SCALE_X;
	for (ptSeg = ptChannel->PHSEG; ptSeg; ptSeg = ptSeg->NEXT)
	{
		ptSeg->X1 = (ptSeg->X1-X1)/SCALE_X;
		ptSeg->Y1 = (ptSeg->Y1-Y1)/SCALE_X;
		ptSeg->X2 = (ptSeg->X2-X1)/SCALE_X;
		ptSeg->Y2 = (ptSeg->Y2-Y1)/SCALE_X;
		if (ptSeg->X1 == 0)
			WestCon = addchain (WestCon, ptSeg->NAME);
		if (ptSeg->X2 == ptChannel->XAB2)
			EastCon = addchain (EastCon, ptSeg->NAME);
			
	}
	for (ptVia = ptChannel->PHVIA; ptVia; ptVia = ptVia->NEXT)
	{
		ptVia->XVIA = (ptVia->XVIA-X1)/SCALE_X;
		ptVia->YVIA = (ptVia->YVIA-Y1)/SCALE_X;
	}

	/* We access the array as array[Width][Height], so we allocate Width lines of Height chars */
	Width  = ptChannel->XAB2 + 1;
	Height = ptChannel->YAB2 + 1;
	Array = (char **) calloc (Width, sizeof (char *));
	for (i=0; i<Width; i++)
		Array[i] = (char *) calloc (Height, sizeof (char));

	/* Loading Array and searching routing Area */
	StartAlu2 = Height;
	EndAlu2   = 0;
	for (ptSeg = ptChannel->PHSEG; ptSeg; ptSeg = ptSeg->NEXT)
	{
		char Layer;
		int  Y2;

		Y1 = ptSeg->Y1;
		Y2 = ptSeg->Y2;
		Layer = (ptSeg->LAYER == ALU1 ? T_ALU1 : T_ALU2);
		if (Y1 == Y2)
		{
			if (Layer == T_ALU2 && Y1 < StartAlu2)
				StartAlu2 = Y1;
			if (Layer == T_ALU2 && Y1 > EndAlu2)
				EndAlu2 = Y1;
			for (i=ptSeg->X1; i<=ptSeg->X2; i++)
				Array[i][Y1] |= Layer;
		}
		else
			for (i=Y1; i<=Y2; i++)
				Array[ptSeg->X1][i] |= Layer;
	}

	/* Loading Vias */
	/* There can be some alu changes so we wont touch these vias */
	Ymin = Height;
	Ymax = 0;
	for (ptVia = ptChannel->PHVIA; ptVia; ptVia = ptVia->NEXT)
	{
		X1 = ptVia->XVIA;
		Y1 = ptVia->YVIA;
		Array[X1][Y1] |= T_VIA|T_ALU1|T_ALU2;
		if ( Y1 < Ymin )
			Ymin = Y1;
		if ( Y1 > Ymax )
			Ymax = Y1;
	}
	/* The Ymin Ymax range mustn't have any via on it (cf OptimizeArray) */
	if (Ymin >= StartAlu2)
		Ymin = 0;
	if (Ymax <= EndAlu2)
		Ymax = Height;
	PRINT (("Ymin = %d, Ymax = %d, StartAlu2 = %d, EndAlu2 = %d\n", Ymin, Ymax, StartAlu2, EndAlu2));
} /* end of LoadArray */ 

/*---------------------------------------------------------\
					SaveArray
\---------------------------------------------------------*/
static phfig_list *SaveArray (X, Y, FileName)
long X, Y;
char *FileName;
{
	phfig_list *ptOptChannel;
	int         FirstNotch, BottomNotchArea, TopNotchArea;
	int         i, j, k, Y1, Y2;
	int         type, First;
	int         AluWidth;
	char       *SegName;

delphfig (FileName);
	ptOptChannel = addphfig (FileName);

	/* Searching the first notch */
	FirstNotch = StartAlu2;
	for (j = Ymin; j < FirstNotch; j++)
	{
		for (i = 0; i<Width-1; i++)
		{
			if (Array[i][j] != EMPTY && Array[i+1][j] != EMPTY)
			{
				FirstNotch = j;
				break;
			}
		}
	}
	PRINT (("First bottom notch : %d\n", FirstNotch));
	BottomNotchArea = max (StartAlu2 - FirstNotch + RULE, RULE+1);
	PRINT (("Place needed for bottom notchs : %d\n", BottomNotchArea));

	/* Searching the last notch */
	FirstNotch = EndAlu2;
	for (j = Ymax; j > FirstNotch; j--)
	{
		for (i = 0; i<Width-1; i++)
		{
			if (Array[i][j] != EMPTY && Array[i+1][j] != EMPTY)
			{
				FirstNotch = j;
				break;
			}
		}
	}
	PRINT (("First top notch : %d\n", FirstNotch));
	TopNotchArea = max (FirstNotch - EndAlu2 + RULE, RULE+1);
	PRINT (("Place needed for top notchs : %d\n", TopNotchArea));

	PRINT (("Won area : %d\n", 
	         StartAlu2 - Ymin - BottomNotchArea + Ymax - EndAlu2 - TopNotchArea));

	BottomNotchArea = StartAlu2 - Ymin - BottomNotchArea;
	if (BottomNotchArea < 0)
		/* in case the first notch is on Ymin, we add RULE to the height, it is not needed */
		BottomNotchArea = 0;
	TopNotchArea    = Ymax - EndAlu2 - TopNotchArea + BottomNotchArea;
	if (TopNotchArea < 0)
		/* in case the last notch is on Ymax, we add RULE to the height, it is not needed */
		TopNotchArea = 0;
	PRINT (("Won bottom space : %d, top space: %d\n", BottomNotchArea, TopNotchArea));

	/* Saving Vias */
	for (i=0; i<Width; i++)
		for (j=0; j<Height; j++)
			if (Array[i][j] & T_VIA)
			{
				Array[i][j] |= T_ALU1|T_ALU2;
				Y1 = j;
				if (Y1 > Ymin)
				{
					if (Y1 <= FirstNotch)
						Y1 -= BottomNotchArea;
					else
						Y1 -= TopNotchArea;
					if (Y1<0)
						Y1 = 0;
				}
				addphvia (ptOptChannel, CONT_VIA, X + i*SCALE_X, Y + Y1*SCALE_X);
			}

	/* We create wires that are at least THREE lambda long.
	 * One lambda: we cross a wire in another direction
	 * Two lambdas: we cross two wire near (no use adding a wire to join them)
	 */

	/* Scanning vertical wires */
	for (k=0; k<2; k++)
	{
		type     = (k==0 ? T_ALU1 : T_ALU2);
		AluWidth = (type == T_ALU1) ? SCALE_X : 2*SCALE_X;
		for (i=0; i<Width; i++)
		{
			for (j=0; j<Height; j++)
			{
				while (j<Height && !(Array[i][j]&type))
					j++;
				if (j<Height)
				{
					First = j;
					j++;
					while (j<Height && (Array[i][j]&type))
						j++;
					j--;
					if (j - First > 1)
					{
						Y1 = First;
						if (Y1 > Ymin)
						{
							if (Y1 <= FirstNotch)
								Y1 -= BottomNotchArea;
							else
								Y1 -= TopNotchArea;
							if (Y1<0)
								Y1 = 0;
						}
						Y2 = j;
						if (Y2 > Ymin)
						{
							if (Y2 <= FirstNotch)
								Y2 -= BottomNotchArea;
							else
								Y2 -= TopNotchArea;
							if (Y2<0)
								Y2 = 0;
						}
						addphseg (ptOptChannel,
					          	type == T_ALU1 ? ALU1:ALU2,
									AluWidth,
					          	X + i*SCALE_X, Y + Y1*SCALE_X,
					          	X + i*SCALE_X, Y + Y2*SCALE_X,
					          	NULL);
					}
				}
			}
		}
	}

	/* Scanning horizontal wires */
	for (k=0; k<2; k++)
	{
		type     = (k==0 ? T_ALU1 : T_ALU2);
		for (j=0; j<Height; j++)
		{
			Y1 = j;
			if (Y1 > Ymin)
			{
				if (Y1 <= FirstNotch)
					Y1 -= BottomNotchArea;
				else
					Y1 -= TopNotchArea;
				if (Y1<0)
					Y1 = 0;
			}
			for (i=0; i<Width; i++)
			{
				while (i<Width && !(Array[i][j]&type))
					i++;
				if (i<Width)
				{
					First = i;
					i++;
					while (i<Width && (Array[i][j]&type))
						i++;
					i--;
					if (i - First > 1)
					{
						if ((type == T_ALU2) ||
						    (((Array[First][j]&T_VIA) || (Array[i][j]&T_VIA)) && (i - First < RULE))
						   )
							AluWidth = 2*SCALE_X;
						else
							AluWidth = SCALE_X;
						if (First == 0)
							
						SegName = ? NULL : ;
						addphseg (ptOptChannel,
									type == T_ALU1 ? ALU1:ALU2,
									AluWidth,
									X + First*SCALE_X, Y + Y1*SCALE_X,
									X + i*SCALE_X,     Y + Y1*SCALE_X,
									NULL);
					}
				}
			}
		}
	}

	for (i=0; i<Width; i++)
		free (Array[i]);
	delphfig (ptChannel);
	defab (ptOptChannel, X, Y, X + (Width-1)*SCALE_X,
                              Y + (Height-TopNotchArea-1)*SCALE_X);
	PRINT (("Saving file\n"));
	return ptOptChannel;
} /* end of SaveArray */
